/*******************************************************
 *  
 *  Programm VControl für Arduino UNO R3
 *  
 *  MEKB e.V. 1932
 *  
 *  Analog Spannung auf A0..A3 einlesen
 *  Wenn die Spannung zwischen 5.0 und 8.0V liegt
 *  wird eine Digitale 1 an entsprechenden DIO ausgegeben 
 *  die LED leuchtet grün ansonten ist der Ausgang Digital 0 
 *  und die LED leuchtet rot
 *  
 *  A0 --> Wert * ( 5.0 / 1023 ) ==> 0.49 mV / Digit
 *  
 *  A0 -> PD6 LOW = LED 2 rot | HIGHT = LED3 grün
 *  A1 -> PD7 LOW = LED 4 rot | HIGHT = LED5 grün
 *  A2 -> PB0 LOW = LED 6 rot | HIGHT = LED7 grün
 *  A3 -> PB1 LOW = LED 8 rot | HIGHT = LED9 grün
 *  
 *  PD1 LED = Lebenslicht
 *  
 *  LCD Display (4 Bit-Mode)
 *  
 *  LCD RS pin to digital pin 12
 *  LCD Enable pin to digital pin 11
 *  LCD D4 pin to digital pin 5
 *  LCD D5 pin to digital pin 4
 *  LCD D6 pin to digital pin 3
 *  LCD D7 pin to digital pin 2
 *  
 *  Flash-Speicher: 3.610 Byte (11%) von 32.256 Byte
 *  SRAM-Speicher :   105 Byte ( 5%) von  2.048 Byte
 *
 *******************************************************/
#include <LiquidCrystal.h>

/*******************************************************
 * Globale Variablen
 *******************************************************/
uint8_t warte = 500;                      // 500ms

uint8_t LED_Ch1 = 9;                      // PB1 = 9
uint8_t LED_Ch2 = 8;                      // PB0 = 8
uint8_t LED_Ch3 = 7;                      // PD7 = 7
uint8_t LED_Ch4 = 6;                      // PD6 = 6

uint16_t lower_Level = 5000;              // unterer Grenzwert [mV]
uint16_t upper_Level = 8500;              // oberer Grenzwert  [mV]

float val_ch1 = 0.00;                     // Puffer für Wandlerwert
float val_ch2 = 0.00;                     // Puffer für Wandlerwert
float val_ch3 = 0.00;                     // Puffer für Wandlerwert
float val_ch4 = 0.00;                     // Puffer für Wandlerwert

LiquidCrystal lcd(12, 11, 5, 4, 3, 2);    // Anschluss LCD 4bit 

/*******************************************************
 * Funktion     : setup
 * 
 * Beschreibung : Diese Funktion läuft einmal zum
 *                Programm START ab
 *                
 *                Hier werden Initialisierungen gemacht
 *                
 *******************************************************/
void setup() {

  // Initialisierung LCDs und setzten der 
  // Auflösung 16 Zeichen und 2 Zeilen
  //lcd.begin(16, 2);
  // Auflösung 20 Zeichen und 2 Zeilen
  lcd.begin(20, 2);

  // Cursor setzten Zeile 0 und Position 1
  lcd.setCursor(0, 1);
  lcd.print("Initialisierung...");
  
  // Definition DIO-Pin als Ausgang
  pinMode( LED_Ch1, OUTPUT);              // LED Ch1 Port PB1 für A0
  pinMode( LED_Ch2, OUTPUT);              // LED Ch2 Port PB0 für A1
  pinMode( LED_Ch3, OUTPUT);              // LED Ch3 Port PD7 für A2
  pinMode( LED_Ch4, OUTPUT);              // LED Ch4 Port PD6 für A3

}

/*******************************************************
 * Funktion     : loop
 * 
 * Beschreibung : Diese Funktion wird ständig wiederholt
 *                
 *                Hier werden die regelmäßigen Aufgaben
 *                aufgerufen
 *                
 *******************************************************/
void loop() {
  // Pause 
  delay ( warte );
  
  // Channel 1..4 scannen
  // Display aktualisieren
  // LED aktualisieren
  for ( uint8_t i_loop = 1; i_loop <= 4; i_loop++ ) {
    int value = scann_channel( i_loop );
    update_lcd( i_loop, value );
    update_led( i_loop, value );
  }
}

/*******************************************************
 * Funktion     : scann_channel
 * 
 * Beschreibung : Analogwerte der Channels einlesen
 * Parameter    : uint8_t channel 1..4
 * Rückgabe     : uint8_t value   0..1023
 *                
 *******************************************************/
int scann_channel ( uint8_t channel ) {
  int val;  // Bereich 0..1023 0x03FF 5V / 1024 = 4,9mV

  if ( channel = 1 ) { 
    val = analogRead( A0 );    // read the input pin }
  } else if ( channel = 2 ) { 
    val = analogRead( A1 );    // read the input pin }
  } else if ( channel = 3 ) { 
    val = analogRead( A2 );    // read the input pin }
  } else if ( channel = 4 ) { 
    val = analogRead( A3 );    // read the input pin }
  } else {
    //
  }
  
  return val;
}

/*******************************************************
 * Funktion     : update_lcd
 * 
 * Beschreibung : aktualisiert die Anzeige des LCD
 * Parameter    : uint8_t channel 1..4
 *                uint8_t value   0..1023
 *      
 * LCD 16x2:
 *       1   2   3   4   5   6   7   8   9   10  11  12  13  14  15  16         
 *     +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
 * Z0  |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
 *     +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
 * Z1  |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
 *     +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
 *  
 *       C   h   1   :   _   _   .   _   C   h   2   :   _   _   .   _ 
 *       C   h   3   :   _   _   .   _   C   h   4   :   _   _   .   _ 
 *    
 * * LCD 20x2:
 *       1   2   3   4   5   6   7   8   9   10  11  12  13  14  15  16  17  18  19  20
 *     +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
 * Z0  |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
 *     +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
 * Z1  |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
 *     +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
 *  
 *       C   h   1   :   _   _   .   _   V       C   h   2   :   _   _   .   _   V    
 *       C   h   3   :   _   _   .   _   V       C   h   4   :   _   _   .   _   V    

 *******************************************************/
void update_lcd ( uint8_t channel, int value ) {
  char StringZ0[17];
  char StringZ1[17];

  if ( channel == 1 ) { 
    float val_ch1 = (value * 5 * 3) / 1000;
  }

  if ( channel == 2 ) { 
    float val_ch2 = (value * 5 * 3) / 1000;
  }

  if ( channel == 3 ) { 
    float val_ch3 = (value * 5 * 3) / 1000;
  }

  if ( channel == 4 ) { 
    float val_ch4 = (value * 5 * 3) / 1000;
  }

  
  sprintf ( StringZ0, "Ch1:%2.1fV Ch2:%2.1fV", val_ch1, val_ch2);
  lcd.setCursor(0, 1);
  lcd.print( StringZ0 );

  sprintf ( StringZ0, "Ch3:%2.1fV Ch4:%2.1fV", val_ch3, val_ch4);
  lcd.setCursor(1, 1);
  lcd.print( StringZ1 );
  return;
}

/*******************************************************
 * Funktion     : update_led
 * 
 * Beschreibung : aktualisiert die Anzeige der LEDs
 * Parameter    : uint8_t channel 1..4
 *                uint8_t value   0..1023
 *                
 *******************************************************/
void update_led ( uint8_t channel, uint8_t value ) {

  if ( channel == 1 ) {              // Auswahl Channel == 1
    if (( value >= lower_Level ) && ( value <= upper_Level )) {
      digitalWrite(LED_Ch1, HIGH);  // Bei HIGH ist die LED "grün" an bei LOW "rot"
    } else {
      digitalWrite(LED_Ch1, LOW);   // Bei LOW ist die LED "rot" an bei HIGH "grün"
    }
  } else if ( channel == 2 ) {      // Auswahl Channel == 2
    if (( value >= lower_Level ) && ( value <= upper_Level )) {
      digitalWrite(LED_Ch2, HIGH);  // Bei HIGH ist die LED "grün" an bei LOW "rot"
    } else {
      digitalWrite(LED_Ch2, LOW);   // Bei LOW ist die LED "rot" an bei HIGH "grün"
    }
  } else if ( channel == 3 ) {      // Auswahl Channel == 3
    if (( value >= lower_Level ) && ( value <= upper_Level )) {
      digitalWrite(LED_Ch3, HIGH);  // Bei HIGH ist die LED "grün" an bei LOW "rot"
    } else {
      digitalWrite(LED_Ch3, LOW);   // Bei LOW ist die LED "rot" an bei HIGH "grün"
    }
  } else if ( channel == 4 ) {      // Auswahl Channel == 4
    if (( value >= lower_Level ) && ( value <= upper_Level )) {
      digitalWrite(LED_Ch4, HIGH);  // Bei HIGH ist die LED "grün" an bei LOW "rot"
    } else {
      digitalWrite(LED_Ch4, LOW);   // Bei LOW ist die LED "rot" an bei HIGH "grün"
    }
  } else {
    //
  }
    
  return;
}

// EOF

